# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/appengine_v1beta/proto/operation.proto

from cloudsdk.google.protobuf import descriptor as _descriptor
from cloudsdk.google.protobuf import message as _message
from cloudsdk.google.protobuf import reflection as _reflection
from cloudsdk.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cloudsdk.google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='google/cloud/appengine_v1beta/proto/operation.proto',
  package='google.appengine.v1beta',
  syntax='proto3',
  serialized_options=b'\n\033com.google.appengine.v1betaB\016OperationProtoP\001Z@google.golang.org/genproto/googleapis/appengine/v1beta;appengine',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n3google/cloud/appengine_v1beta/proto/operation.proto\x12\x17google.appengine.v1beta\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/api/annotations.proto\"\xbe\x02\n\x17OperationMetadataV1Beta\x12\x0e\n\x06method\x18\x01 \x01(\t\x12/\n\x0binsert_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04user\x18\x04 \x01(\t\x12\x0e\n\x06target\x18\x05 \x01(\t\x12\x19\n\x11\x65phemeral_message\x18\x06 \x01(\t\x12\x0f\n\x07warning\x18\x07 \x03(\t\x12W\n\x17\x63reate_version_metadata\x18\x08 \x01(\x0b\x32\x34.google.appengine.v1beta.CreateVersionMetadataV1BetaH\x00\x42\x11\n\x0fmethod_metadata\"5\n\x1b\x43reateVersionMetadataV1Beta\x12\x16\n\x0e\x63loud_build_id\x18\x01 \x01(\tBq\n\x1b\x63om.google.appengine.v1betaB\x0eOperationProtoP\x01Z@google.golang.org/genproto/googleapis/appengine/v1beta;appengineb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_OPERATIONMETADATAV1BETA = _descriptor.Descriptor(
  name='OperationMetadataV1Beta',
  full_name='google.appengine.v1beta.OperationMetadataV1Beta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='method', full_name='google.appengine.v1beta.OperationMetadataV1Beta.method', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='insert_time', full_name='google.appengine.v1beta.OperationMetadataV1Beta.insert_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end_time', full_name='google.appengine.v1beta.OperationMetadataV1Beta.end_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user', full_name='google.appengine.v1beta.OperationMetadataV1Beta.user', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='target', full_name='google.appengine.v1beta.OperationMetadataV1Beta.target', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ephemeral_message', full_name='google.appengine.v1beta.OperationMetadataV1Beta.ephemeral_message', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='warning', full_name='google.appengine.v1beta.OperationMetadataV1Beta.warning', index=6,
      number=7, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='create_version_metadata', full_name='google.appengine.v1beta.OperationMetadataV1Beta.create_version_metadata', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='method_metadata', full_name='google.appengine.v1beta.OperationMetadataV1Beta.method_metadata',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=144,
  serialized_end=462,
)


_CREATEVERSIONMETADATAV1BETA = _descriptor.Descriptor(
  name='CreateVersionMetadataV1Beta',
  full_name='google.appengine.v1beta.CreateVersionMetadataV1Beta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cloud_build_id', full_name='google.appengine.v1beta.CreateVersionMetadataV1Beta.cloud_build_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=464,
  serialized_end=517,
)

_OPERATIONMETADATAV1BETA.fields_by_name['insert_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_OPERATIONMETADATAV1BETA.fields_by_name['end_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_OPERATIONMETADATAV1BETA.fields_by_name['create_version_metadata'].message_type = _CREATEVERSIONMETADATAV1BETA
_OPERATIONMETADATAV1BETA.oneofs_by_name['method_metadata'].fields.append(
  _OPERATIONMETADATAV1BETA.fields_by_name['create_version_metadata'])
_OPERATIONMETADATAV1BETA.fields_by_name['create_version_metadata'].containing_oneof = _OPERATIONMETADATAV1BETA.oneofs_by_name['method_metadata']
DESCRIPTOR.message_types_by_name['OperationMetadataV1Beta'] = _OPERATIONMETADATAV1BETA
DESCRIPTOR.message_types_by_name['CreateVersionMetadataV1Beta'] = _CREATEVERSIONMETADATAV1BETA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

OperationMetadataV1Beta = _reflection.GeneratedProtocolMessageType('OperationMetadataV1Beta', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONMETADATAV1BETA,
  '__module__' : 'google.cloud.appengine_v1beta.proto.operation_pb2'
  ,
  '__doc__': """Metadata for the given
  [google.longrunning.Operation][google.longrunning.Operation].
  
  Attributes:
      method:
          API method that initiated this operation. Example:
          ``google.appengine.v1beta.Versions.CreateVersion``.
          @OutputOnly
      insert_time:
          Time that this operation was created.  @OutputOnly
      end_time:
          Time that this operation completed.  @OutputOnly
      user:
          User who requested this operation.  @OutputOnly
      target:
          Name of the resource that this operation is acting on.
          Example: ``apps/myapp/services/default``.  @OutputOnly
      ephemeral_message:
          Ephemeral message that may change every time the operation is
          polled. @OutputOnly
      warning:
          Durable messages that persist on every operation poll.
          @OutputOnly
      method_metadata:
          Metadata specific to the type of operation in progress.
          @OutputOnly
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1beta.OperationMetadataV1Beta)
  })
_sym_db.RegisterMessage(OperationMetadataV1Beta)

CreateVersionMetadataV1Beta = _reflection.GeneratedProtocolMessageType('CreateVersionMetadataV1Beta', (_message.Message,), {
  'DESCRIPTOR' : _CREATEVERSIONMETADATAV1BETA,
  '__module__' : 'google.cloud.appengine_v1beta.proto.operation_pb2'
  ,
  '__doc__': """Metadata for the given
  [google.longrunning.Operation][google.longrunning.Operation] during a 
  [google.appengine.v1beta.CreateVersionRequest][google.appengine.v1beta
  .CreateVersionRequest].
  
  Attributes:
      cloud_build_id:
          The Cloud Build ID if one was created as part of the version
          create. @OutputOnly
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1beta.CreateVersionMetadataV1Beta)
  })
_sym_db.RegisterMessage(CreateVersionMetadataV1Beta)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
