# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create an enterprise-certificate configuration file for Windows."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.auth import enterprise_certificate_config
from googlecloudsdk.command_lib.auth.flags import AddCommonEnterpriseCertConfigFlags


class CreateWindowsConfig(base.CreateCommand):
  """Create an enterprise-certificate configuration file for Windows.

  This command creates a configuration file used by gcloud to use the
  enterprise-certificate-proxy component for mTLS.
  """

  detailed_help = {
      'EXAMPLES': textwrap.dedent(
          """\
          To create a credential configuration run:

            $ {command} --issuer=$CERT_ISSUER --store=$STORE --provider=$PROVIDER
          """
      ),
  }

  @classmethod
  def Args(cls, parser):
    AddCommonEnterpriseCertConfigFlags(parser)
    parser.add_argument(
        '--issuer',
        help='The certificate issuer.',
        required=True,
    )

    parser.add_argument(
        '--store',
        help='The Windows secure store.',
        required=True,
    )

    parser.add_argument(
        '--provider',
        help='The Windows secure store provider.',
        required=True,
    )

  def Run(self, args):
    enterprise_certificate_config.create_config(
        args, enterprise_certificate_config.ConfigType.MYSTORE
    )
