# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/appengine_v1/proto/location.proto

from cloudsdk.google.protobuf import descriptor as _descriptor
from cloudsdk.google.protobuf import message as _message
from cloudsdk.google.protobuf import reflection as _reflection
from cloudsdk.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='google/cloud/appengine_v1/proto/location.proto',
  package='google.appengine.v1',
  syntax='proto3',
  serialized_options=b'\n\027com.google.appengine.v1B\rLocationProtoP\001Z<google.golang.org/genproto/googleapis/appengine/v1;appengine',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.google/cloud/appengine_v1/proto/location.proto\x12\x13google.appengine.v1\x1a\x1cgoogle/api/annotations.proto\"b\n\x10LocationMetadata\x12&\n\x1estandard_environment_available\x18\x02 \x01(\x08\x12&\n\x1e\x66lexible_environment_available\x18\x04 \x01(\x08\x42h\n\x17\x63om.google.appengine.v1B\rLocationProtoP\x01Z<google.golang.org/genproto/googleapis/appengine/v1;appengineb\x06proto3'
  ,
  dependencies=[google_dot_api_dot_annotations__pb2.DESCRIPTOR,])




_LOCATIONMETADATA = _descriptor.Descriptor(
  name='LocationMetadata',
  full_name='google.appengine.v1.LocationMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='standard_environment_available', full_name='google.appengine.v1.LocationMetadata.standard_environment_available', index=0,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flexible_environment_available', full_name='google.appengine.v1.LocationMetadata.flexible_environment_available', index=1,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=101,
  serialized_end=199,
)

DESCRIPTOR.message_types_by_name['LocationMetadata'] = _LOCATIONMETADATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LocationMetadata = _reflection.GeneratedProtocolMessageType('LocationMetadata', (_message.Message,), {
  'DESCRIPTOR' : _LOCATIONMETADATA,
  '__module__' : 'google.cloud.appengine_v1.proto.location_pb2'
  ,
  '__doc__': """Metadata for the given
  [google.cloud.location.Location][google.cloud.location.Location].
  
  Attributes:
      standard_environment_available:
          App Engine standard environment is available in the given
          location.  @OutputOnly
      flexible_environment_available:
          App Engine flexible environment is available in the given
          location.  @OutputOnly
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.LocationMetadata)
  })
_sym_db.RegisterMessage(LocationMetadata)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
