# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The super-group for the Cloud NetApp Files CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base

DETAILED_HELP = {
    'DESCRIPTION':
        """\
      The gcloud netapp command group lets you create, configure and manipulate
      Cloud NetApp Files resources such as Volumes, Storage Pools, etc.

      With Cloud NetApp Files, you can take advantage of Google Cloud Platform's
      scale, performance and value to manage and run on-prem NetApp Files
      solutions on Google infrastructure, so you don't have to modernize your
      file workloads, and allow for legacy feature support.

      More information on Cloud NetApp Files can be found here:
      https://cloud.google.com/netapp/docs/reference/rest
      """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Netapp(base.Group):
  """Create and manipulate Cloud NetApp Files resources."""
  detailed_help = DETAILED_HELP
  category = base.STORAGE_CATEGORY
